// 双向链表的插入。使用幽灵循环，目的是为了分段描述代码中的循环不变式。
// 要点：
//     通过循环在链表中找到指定的插入位置，将新节点插入到链表中
// 关键字：双向链表
#include <stddef.h>
#include <stdlib.h>

#define N 100 

typedef struct DLNode {
  struct DLNode *r;
  struct DLNode *l;
  int data2;
}dnode;
///////////////////////////////声明结构体自引用l，r域构成双向链表//////////////////////////////
//@ shape l, r: dlist;

///////////////////////////////////////声明逻辑变量//////////////////////////////////////////
//@ logic dnode* oldp;//用于记录函数入口处的指针p

/////////////////////////////////////insert函数的验证////////////////////////////////////////
// 函数协议：
// 函数前条件：p为空或者指向长为n的双向链表
// 函数后条件：返回值指向长为n+1的双向链表

/*@ 
  requires     
      p == \null && n == 0 && oldp == p //函数入口处的链表为空
  || 
      p != \null && \dlist(p) && \length(p,r) == n 
      && n >= 1 && 1 <= i <= n && n <= N 
      && oldp == p
      ;
  assigns oldp;
  exits 
      \exit_status == 1
      ;
  ensures 
      \dlist(\result) && \length(\result,r) == n+1 
      && (oldp == \null || oldp == \result)
      ;
*/
dnode* insert(dnode* p, int const n, int i)
//@ shape dnode forward: r;//定义访问链表的主方向为r指针
{
    dnode *q;
    dnode *s;
    int j;
    //@ ghost dnode *t; 
    //@ ghost int k; //声明幽灵变量

    if (p == NULL) {
        p = (dnode*)malloc(sizeof(dnode));
        if (p == NULL) {
            exit(1);
        }
        p->l = NULL;
        p->r = NULL;
    } else {
        //@ ghost t = p; 
        //@ ghost k = 0;
        /*@ 
          loop invariant
              n >= 1 && 1 <= i <= n && n <= N 
              && oldp == p 
              && p != \null && \dangling(s) && \dangling(q) 
              && \dlist(p) && \length(p,r) == n //以p为头节点的双向链表长度为n
              && t == p->(r: k) && 0 <= k < n; //t位于该链表中第k+1个位置，并给出k的范围
        */
        /*@
            ghost while(t->r != NULL) {
                t = t->r;
                k = k + 1;
                }
        */
        
        j = 1;
        q = p;
        /*@ 
          loop invariant 
              n >= 1 && 1 <= i <= n && n <= N && 1 <= j <= i 
              && oldp == p && p != \null 
              && \dangling(s) 
              && p->l == \null 
              && \dlist_seg(p, q) && q == p->(r:j-1) 
              //p指向的节点到q指向的节点构成双向链表表段，q在该表段中位于第j个位置
              && \dlist_seg(q, t) && t == q->(r:k-j+1) 
              //q指向的节点到t指向的节点构成双向链表表段，t在该表段中位于第k-j+2个位置
              && t->r == \null && k == n - 1 //上面幽灵循环不变式出口的状态代到下面来
              ;
        */
        while ((j < i) && (q->r != NULL)) {
            q = q->r;
            j = j + 1;
        }
        s = (dnode*)malloc(sizeof(dnode));
        if (s == NULL) {
            exit(1);
        }
        s->l = q;
        s->r = q->r;
        if (q->r != NULL) {
            q->r->l = s;
        }
        q->r = s;
    }
    return p;
}
